package cs2.introml.tests;

import java.io.IOException;
import java.util.Map;
import java.util.TreeMap;

import cs2.introml.Example;
import cs2.introml.FeatureEntry;
import cs2.introml.HiddenPatternDataSet;
import cs2.introml.LibSVMDataSet;
import cs2.introml.SparseExample;
import cs2.introml.HiddenPatternDataSet.PatternException;
import junit.framework.TestCase;

public class P3Tests extends TestCase {
	
public void testHiddenPattern0() throws PatternException {
		
		final int numFeatures = 2;
		final int numExamples = 10;
		final int magicFeature = 1;
		final double magicValue = 3.14;
		final double otherValue = 0.;
		final Object label = "foo";
		
		final Map<Integer, Double> m = new TreeMap<Integer, Double>();
		final HiddenPatternDataSet ds = new HiddenPatternDataSet( numFeatures, numExamples );
		
		m.clear();
		m.put( magicFeature, magicValue );
		ds.addPattern( label, m );
		
		assertEquals( numFeatures, ds.getNumFeatures() );
		assertEquals( numExamples, ds.getNumExamples() );
		
		for ( Example e : ds ) {
			
			for ( FeatureEntry fe : e ) {
				if ( fe.getNumber() == magicFeature ) {
					assertEquals( magicValue, fe.getFeature() );
				} else {
					assertEquals( otherValue, fe.getFeature() );
				}
			}
			
			for ( FeatureEntry fe : ((SparseExample) e).sparseFeatures() ) {
				assertEquals( magicFeature, fe.getNumber() );
				assertEquals( magicValue, fe.getFeature() );
			}
			
		}
		
		assertTrue( ds.toString().equals(
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n" +
			"foo 2:3.14" + "\n"	
		) );
		
	}

	public void testHiddenPattern0_2() throws PatternException {
		
		final Map<Integer, Double> m = new TreeMap<Integer, Double>();
		final HiddenPatternDataSet ds = new HiddenPatternDataSet( 10, 6 );
		
		m.clear();
		m.put( 1, 1. );
		m.put( 2, 3.14 );
		ds.addPattern( "cat", m );
		
		m.clear();
		m.put( 1, 2. );
		m.put( 4, 2.7 );
		ds.addPattern( "dog", m );
		
		assertTrue( ds.toString().equals(
			"cat 2:1.0 3:3.14" + "\n" +
			"dog 2:2.0 5:2.7" + "\n" +
			"cat 2:1.0 3:3.14" + "\n" +
			"dog 2:2.0 5:2.7" + "\n" +
			"cat 2:1.0 3:3.14" + "\n" +
			"dog 2:2.0 5:2.7" + "\n"
		) );
		
	}
	
	public void testHiddenPatternRand() throws PatternException {
		
		final int numFeatures = 2;
		final int numExamples = 10;
		final int magicFeature = 1;
		final double magicValue = 3.14;
		final Object label = "foo";
		
		final Map<Integer, Double> m = new TreeMap<Integer, Double>();
		final HiddenPatternDataSet ds = new HiddenPatternDataSet( numFeatures, numExamples, 42, -2, -1 );
		
		m.clear();
		m.put( magicFeature, magicValue );
		ds.addPattern( label, m );
		
		assertEquals( numFeatures, ds.getNumFeatures() );
		assertEquals( numExamples, ds.getNumExamples() );
		
		for ( Example e : ds ) {
			
			for ( FeatureEntry fe : e ) {
				if ( fe.getNumber() == magicFeature ) {
					assertEquals( magicValue, fe.getFeature() );
				} else {
					assertTrue( fe.getFeature() >= -2 && fe.getFeature() <= -1 );
				}
			}
			
		}
		
	}
	
	public void testLibSVM() throws IOException {
		
		final int numFeatures = 5;
		final int numExamples = 5;
		final int magicFeature = 4;
		final double magicValue = 1.;
		final double otherValue = 0.5;
		final Object label = 1;
		
		final LibSVMDataSet ds = new LibSVMDataSet( DataSetTest.class.getResourceAsStream( "data/libsvm.data" ), numFeatures, LibSVMDataSet.IntegerInterpreter );
		
		assertEquals( numFeatures, ds.getNumFeatures() );
		assertEquals( numExamples, ds.getNumExamples() );
		
		int eCounter = 0;
		for ( Example e : ds ) {
			
			eCounter++;
			assertEquals( label, e.getResult() );
			
			for ( FeatureEntry fe : ((SparseExample) e).sparseFeatures() ) {
				
				if ( fe.getNumber() == magicFeature ) {
					
					assertEquals( magicValue, fe.getFeature() );
					
				} else {
					
					assertTrue( fe.getNumber() < eCounter );
					assertEquals( otherValue, fe.getFeature() );
					
				}
				
			}
			
		}
		
		assertTrue( ds.toString().equals( 
			"1 5:1.0" + "\n" +
			"1 1:0.5 5:1.0" + "\n" +
			"1 1:0.5 2:0.5 5:1.0" + "\n" +
			"1 1:0.5 2:0.5 3:0.5 5:1.0" + "\n" +
			"1 1:0.5 2:0.5 3:0.5 4:0.5 5:1.0" + "\n"
		) );
		
	}

}
